def check (cmd ,mf ):
    print ("CTYPES USERS",list (mf .getReferers ("ctypes")))
    m =mf .findNode ("ctypes")
    if m is None or m .filename is None :
        return None 

    return {"prescripts":["py2app.bootstrap.ctypes_setup"]}



import os 
import sys 

from modulegraph .util import imp_find_module 

try :
    from cStringIO import StringIO 
except ImportError :
    from io import StringIO 

try :
    set 
except NameError :
    from sets import Set as set 

try :
    basestring 
except NameError :
    basestring =str 


def check (cmd ,mf ):
    m =mf .findNode ("Image")or mf .findNode ("PIL.Image")
    if m is None or m .filename is None :
        return None 

    have_PIL =bool (mf .findNode ("PIL.Image"))

    plugins =set ()
    visited =set ()



    for folder in sys .path :
        if not isinstance (folder ,basestring ):
            continue 

        for extra in ("","PIL"):
            folder =os .path .realpath (os .path .join (folder ,extra ))
            if (not os .path .isdir (folder ))or (folder in visited ):
                continue 
            for fn in os .listdir (folder ):
                if not fn .endswith ("ImagePlugin.py"):
                    continue 

                mod ,ext =os .path .splitext (fn )
                try :
                    sys .path .insert (0 ,folder )
                    imp_find_module (mod )
                    del sys .path [0 ]
                except ImportError :
                    pass 
                else :
                    plugins .add (mod )
        visited .add (folder )
    s =StringIO ("_recipes_pil_prescript(%r)\n"%list (plugins ))
    print (plugins )
    plugins =set ()

    for plugin in plugins :
        if have_PIL :
            mf .implyNodeReference (m ,"PIL."+plugin )
        else :
            mf .implyNodeReference (m ,plugin )

    mf .removeReference (m ,"FixTk")


    sip =mf .findNode ("SpiderImagePlugin")
    if sip is not None :
        mf .removeReference (sip ,"ImageTk")





    sip =mf .findNode ("PIL.ImageQt")
    if sip is not None :
        mf .removeReference (sip ,"PyQt5")
        mf .removeReference (sip ,"PyQt5.QtGui")
        mf .removeReference (sip ,"PyQt5.QtCore")

        mf .removeReference (sip ,"PyQt4")
        mf .removeReference (sip ,"PyQt4.QtGui")
        mf .removeReference (sip ,"PyQt4.QtCore")
        pass 

    imagefilter =mf .findNode ("PIL.ImageFilter")
    if imagefilter is not None :




        mf .removeReference (imagefilter ,"numpy")

    image =mf .findNode ("PIL.Image")
    if image is not None :


        mf .removeReference (image ,"numpy")

    return {
    "prescripts":["py2app.recipes.PIL.prescript",s ],
    "include":"PIL.JpegPresets",
    "flatpackages":[os .path .dirname (m .filename )],
    }



import sys 

import py2app 

__all__ =["infoPlistDict"]


def infoPlistDict (CFBundleExecutable ,plist ={}):
    CFBundleExecutable =CFBundleExecutable 
    version =sys .version [:3 ]
    pdict =dict (
    CFBundleDevelopmentRegion ="English",
    CFBundleDisplayName =plist .get ("CFBundleName",CFBundleExecutable ),
    CFBundleExecutable =CFBundleExecutable ,
    CFBundleIconFile =CFBundleExecutable ,
    CFBundleIdentifier ="org.pythonmac.unspecified.%s"
    %("".join (CFBundleExecutable .split ()),),
    CFBundleInfoDictionaryVersion ="6.0",
    CFBundleName =CFBundleExecutable ,
    CFBundlePackageType ="APPL",
    CFBundleShortVersionString =plist .get ("CFBundleVersion","0.0"),
    CFBundleSignature ="????",
    CFBundleVersion ="0.0",
    LSHasLocalizedDisplayName =False ,
    NSAppleScriptEnabled =False ,
    NSHumanReadableCopyright ="Copyright not specified",
    NSMainNibFile ="MainMenu",
    NSPrincipalClass ="NSApplication",
    PyMainFileNames =["__boot__"],
    PyResourcePackages =[],
    PyRuntimeLocations =[
    (s %version )
    for s in [
    (
    "@executable_path/../Frameworks/Python.framework"
    "/Versions/%s/Python"
    ),
    "~/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/Network/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/System/Library/Frameworks/Python.framework/Versions/%s/Python",
    ]
    ],
    )
    pdict .update (plist )
    pythonInfo =pdict .setdefault ("PythonInfoDict",{})
    pythonInfo .update (
    dict (
    PythonLongVersion =sys .version ,
    PythonShortVersion =sys .version [:3 ],
    PythonExecutable =sys .executable ,
    )
    )
    py2appInfo =pythonInfo .setdefault ("py2app",{})
    py2appInfo .update (
    dict (
    version =py2app .__version__ ,
    template ="app",
    )
    )
    return pdict 



import sys 

import py2app 

__all__ =["infoPlistDict"]


def infoPlistDict (CFBundleExecutable ,plist =None ):
    if plist is None :
        plist ={}
    CFBundleExecutable =CFBundleExecutable 
    NSPrincipalClass ="".join (CFBundleExecutable .split ())
    version =sys .version [:3 ]
    pdict ={
    "CFBundleDevelopmentRegion":"English",
    "CFBundleDisplayName":plist .get ("CFBundleName",CFBundleExecutable ),
    "CFBundleExecutable":CFBundleExecutable ,
    "CFBundleIconFile":CFBundleExecutable ,
    "CFBundleIdentifier":"org.pythonmac.unspecified.%s"%(NSPrincipalClass ,),
    "CFBundleInfoDictionaryVersion":"6.0",
    "CFBundleName":CFBundleExecutable ,
    "CFBundlePackageType":"BNDL",
    "CFBundleShortVersionString":plist .get ("CFBundleVersion","0.0"),
    "CFBundleSignature":"????",
    "CFBundleVersion":"0.0",
    "LSHasLocalizedDisplayName":False ,
    "NSAppleScriptEnabled":False ,
    "NSHumanReadableCopyright":"Copyright not specified",
    "NSMainNibFile":"MainMen",
    "NSPrincipalClass":NSPrincipalClass ,
    "PyMainFileNames":["__boot__"],
    "PyResourcePackages":[
    (s %version )
    for s in [
    "lib/python%s",
    "lib/python%s/lib-dynload",
    "lib/python%s/site-packages.zip",
    ]
    ]
    +["lib/python%s.zip"%version .replace (".","")],
    "PyRuntimeLocations":[
    (s %version )
    for s in [
    (
    "@executable_path/../Frameworks/Python.framework"
    "/Versions/%s/Python"
    ),
    "~/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/Network/Library/Frameworks/Python.framework/Versions/%s/Python",
    "/System/Library/Frameworks/Python.framework/Versions/%s/Python",
    ]
    ],
    }
    pdict .update (plist )
    pythonInfo =pdict .setdefault ("PythonInfoDict",{})
    pythonInfo .update (
    {
    "PythonLongVersion":sys .version ,
    "PythonShortVersion":sys .version [:3 ],
    "PythonExecutable":sys .executable ,
    }
    )
    py2appInfo =pythonInfo .setdefault ("py2app",{})
    py2appInfo .update ({"version":py2app .__version__ ,"template":"bundle"})
    return pdict 



"""
altgraph.GraphStat - Functions providing various graph statistics
=================================================================
"""


def degree_dist (graph ,limits =(0 ,0 ),bin_num =10 ,mode ="out"):
    """
    Computes the degree distribution for a graph.

    Returns a list of tuples where the first element of the tuple is the
    center of the bin representing a range of degrees and the second element
    of the tuple are the number of nodes with the degree falling in the range.

    Example::

        ....
    """

    deg =[]
    if mode =="inc":
        get_deg =graph .inc_degree 
    else :
        get_deg =graph .out_degree 

    for node in graph :
        deg .append (get_deg (node ))

    if not deg :
        return []

    results =_binning (values =deg ,limits =limits ,bin_num =bin_num )

    return results 


_EPS =1.0 /(2.0 **32 )


def _binning (values ,limits =(0 ,0 ),bin_num =10 ):
    """
    Bins data that falls between certain limits, if the limits are (0, 0) the
    minimum and maximum values are used.

    Returns a list of tuples where the first element of the tuple is the
    center of the bin and the second element of the tuple are the counts.
    """
    if limits ==(0 ,0 ):
        min_val ,max_val =min (values )-_EPS ,max (values )+_EPS 
    else :
        min_val ,max_val =limits 


    bin_size =(max_val -min_val )/float (bin_num )
    bins =[0 ]*(bin_num )


    for value in values :
        try :
            if (value -min_val )>=0 :
                index =int ((value -min_val )/float (bin_size ))
                bins [index ]+=1 
        except IndexError :
            pass 


    result =[]
    center =(bin_size /2 )+min_val 
    for i ,y in enumerate (bins ):
        x =center +bin_size *i 
        result .append ((x ,y ))

    return result 



def _recipes_pil_prescript (plugins ):
    try :
        import Image 

        have_PIL =False 
    except ImportError :
        from PIL import Image 

        have_PIL =True 

    import sys 

    def init ():
        if Image ._initialized >=2 :
            return 

        if have_PIL :
            try :
                import PIL .JpegPresets 

                sys .modules ["JpegPresets"]=PIL .JpegPresets 
            except ImportError :
                pass 

        for plugin in plugins :
            try :
                if have_PIL :
                    try :


                        m =__import__ ("PIL."+plugin ,globals (),locals (),[])
                        m =getattr (m ,plugin )
                        sys .modules [plugin ]=m 
                        continue 
                    except ImportError :
                        pass 

                __import__ (plugin ,globals (),locals (),[])
            except ImportError :
                print ("Image: failed to import")

        if Image .OPEN or Image .SAVE :
            Image ._initialized =2 
            return 1 

    Image .init =init 



import os 
import shutil 
import stat 
import struct 
import sys 

from macholib import mach_o 

MAGIC =[
struct .pack ("!L",getattr (mach_o ,"MH_"+_ ))
for _ in ["MAGIC","CIGAM","MAGIC_64","CIGAM_64"]
]
FAT_MAGIC_BYTES =struct .pack ("!L",mach_o .FAT_MAGIC )
MAGIC_LEN =4 
STRIPCMD =["/usr/bin/strip","-x","-S","-"]

try :
    unicode 
except NameError :
    unicode =str 


def fsencoding (s ,encoding =sys .getfilesystemencoding ()):
    """
    Ensure the given argument is in filesystem encoding (not unicode)
    """
    if isinstance (s ,unicode ):
        s =s .encode (encoding )
    return s 


def move (src ,dst ):
    """
    move that ensures filesystem encoding of paths
    """
    shutil .move (fsencoding (src ),fsencoding (dst ))


def copy2 (src ,dst ):
    """
    copy2 that ensures filesystem encoding of paths
    """
    shutil .copy2 (fsencoding (src ),fsencoding (dst ))


def flipwritable (fn ,mode =None ):
    """
    Flip the writability of a file and return the old mode. Returns None
    if the file is already writable.
    """
    if os .access (fn ,os .W_OK ):
        return None 
    old_mode =os .stat (fn ).st_mode 
    os .chmod (fn ,stat .S_IWRITE |old_mode )
    return old_mode 


class fileview (object ):
    """
    A proxy for file-like objects that exposes a given view of a file
    """

    def __init__ (self ,fileobj ,start ,size ):
        self ._fileobj =fileobj 
        self ._start =start 
        self ._end =start +size 

    def __repr__ (self ):
        return "<fileview [%d, %d] %r>"%(self ._start ,self ._end ,self ._fileobj )

    def tell (self ):
        return self ._fileobj .tell ()-self ._start 

    def _checkwindow (self ,seekto ,op ):
        if not (self ._start <=seekto <=self ._end ):
            raise IOError (
            "%s to offset %d is outside window [%d, %d]"
            %(op ,seekto ,self ._start ,self ._end )
            )

    def seek (self ,offset ,whence =0 ):
        seekto =offset 
        if whence ==0 :
            seekto +=self ._start 
        elif whence ==1 :
            seekto +=self ._fileobj .tell ()
        elif whence ==2 :
            seekto +=self ._end 
        else :
            raise IOError ("Invalid whence argument to seek: %r"%(whence ,))
        self ._checkwindow (seekto ,"seek")
        self ._fileobj .seek (seekto )

    def write (self ,bytes ):
        here =self ._fileobj .tell ()
        self ._checkwindow (here ,"write")
        self ._checkwindow (here +len (bytes ),"write")
        self ._fileobj .write (bytes )

    def read (self ,size =sys .maxsize ):
        if size <0 :
            raise ValueError (
            "Invalid size %s while reading from %s",size ,self ._fileobj 
            )
        here =self ._fileobj .tell ()
        self ._checkwindow (here ,"read")
        bytes =min (size ,self ._end -here )
        return self ._fileobj .read (bytes )


def mergecopy (src ,dest ):
    """
    copy2, but only if the destination isn't up to date
    """
    if os .path .exists (dest )and os .stat (dest ).st_mtime >=os .stat (src ).st_mtime :
        return 

    copy2 (src ,dest )


def mergetree (src ,dst ,condition =None ,copyfn =mergecopy ,srcbase =None ):
    """
    Recursively merge a directory tree using mergecopy().
    """
    src =fsencoding (src )
    dst =fsencoding (dst )
    if srcbase is None :
        srcbase =src 
    names =map (fsencoding ,os .listdir (src ))
    try :
        os .makedirs (dst )
    except OSError :
        pass 
    errors =[]
    for name in names :
        srcname =os .path .join (src ,name )
        dstname =os .path .join (dst ,name )
        if condition is not None and not condition (srcname ):
            continue 
        try :
            if os .path .islink (srcname ):
                realsrc =os .readlink (srcname )
                os .symlink (realsrc ,dstname )
            elif os .path .isdir (srcname ):
                mergetree (
                srcname ,
                dstname ,
                condition =condition ,
                copyfn =copyfn ,
                srcbase =srcbase ,
                )
            else :
                copyfn (srcname ,dstname )
        except (IOError ,os .error )as why :
            errors .append ((srcname ,dstname ,why ))
    if errors :
        raise IOError (errors )


def sdk_normalize (filename ):
    """
    Normalize a path to strip out the SDK portion, normally so that it
    can be decided whether it is in a system path or not.
    """
    if filename .startswith ("/Developer/SDKs/"):
        pathcomp =filename .split ("/")
        del pathcomp [1 :4 ]
        filename ="/".join (pathcomp )
    return filename 


NOT_SYSTEM_FILES =[]


def in_system_path (filename ):
    """
    Return True if the file is in a system path
    """
    fn =sdk_normalize (os .path .realpath (filename ))
    if fn .startswith ("/usr/local/"):
        return False 
    elif fn .startswith ("/System/")or fn .startswith ("/usr/"):
        if fn in NOT_SYSTEM_FILES :
            return False 
        return True 
    else :
        return False 


def has_filename_filter (module ):
    """
    Return False if the module does not have a filename attribute
    """
    return getattr (module ,"filename",None )is not None 


def get_magic ():
    """
    Get a list of valid Mach-O header signatures, not including the fat header
    """
    return MAGIC 


def is_platform_file (path ):
    """
    Return True if the file is Mach-O
    """
    if not os .path .exists (path )or os .path .islink (path ):
        return False 

    with open (path ,"rb")as fileobj :
        bytes =fileobj .read (MAGIC_LEN )
        if bytes ==FAT_MAGIC_BYTES :

            fileobj .seek (0 )
            header =mach_o .fat_header .from_fileobj (fileobj ,_endian_ =">")
            if header .nfat_arch <1 :
                return False 

            arch =mach_o .fat_arch .from_fileobj (fileobj ,_endian_ =">")
            fileobj .seek (arch .offset )

            bytes =fileobj .read (MAGIC_LEN )
    for magic in MAGIC :
        if bytes ==magic :
            return True 
    return False 


def iter_platform_files (dst ):
    """
    Walk a directory and yield each full path that is a Mach-O file
    """
    for root ,_dirs ,files in os .walk (dst ):
        for fn in files :
            fn =os .path .join (root ,fn )
            if is_platform_file (fn ):
                yield fn 


def strip_files (files ,argv_max =(256 *1024 )):
    """
    Strip a list of files
    """
    tostrip =[(fn ,flipwritable (fn ))for fn in files ]
    while tostrip :
        cmd =list (STRIPCMD )
        flips =[]
        pathlen =sum (len (s )+1 for s in cmd )
        while pathlen <argv_max :
            if not tostrip :
                break 
            added ,flip =tostrip .pop ()
            pathlen +=len (added )+1 
            cmd .append (added )
            flips .append ((added ,flip ))
        else :
            cmd .pop ()
            tostrip .append (flips .pop ())
        os .spawnv (os .P_WAIT ,cmd [0 ],cmd )
        for args in flips :
            flipwritable (*args )



"""
Internal helpers for basic commandline tools
"""
from __future__ import absolute_import ,print_function 

import os 
import sys 

from macholib .util import is_platform_file 


def check_file (fp ,path ,callback ):
    if not os .path .exists (path ):
        print (
        "%s: %s: No such file or directory"%(sys .argv [0 ],path ),file =sys .stderr 
        )
        return 1 

    try :
        is_plat =is_platform_file (path )

    except IOError as msg :
        print ("%s: %s: %s"%(sys .argv [0 ],path ,msg ),file =sys .stderr )
        return 1 

    else :
        if is_plat :
            callback (fp ,path )
    return 0 


def main (callback ):
    args =sys .argv [1 :]
    name =os .path .basename (sys .argv [0 ])
    err =0 

    if not args :
        print ("Usage: %s filename..."%(name ,),file =sys .stderr )
        return 1 

    for base in args :
        if os .path .isdir (base ):
            for root ,_dirs ,files in os .walk (base ):
                for fn in files :
                    err |=check_file (sys .stdout ,os .path .join (root ,fn ),callback )
        else :
            err |=check_file (sys .stdout ,base ,callback )

    return err 



"""
builds Mac OS X application bundles from Python scripts

New keywords for distutils' setup function specify what to build:

    app
        list of scripts to convert into gui app bundles

py2app options, to be specified in the options keyword to the setup function:

    optimize - string or int (0, 1, or 2)

    includes - list of module names to include
    packages - list of packages to include with subpackages
    ignores - list of modules to ignore if they are not found
    excludes - list of module names to exclude
    dylib_excludes - list of dylibs and/or frameworks to exclude
    resources - list of additional files and folders to include
    plist - Info.plist template file, dict, or plistlib.Plist
    dist_dir - directory where to build the final files

Items in the macosx list can also be
dictionaries to further customize the build process.  The following
keys in the dictionary are recognized, most are optional:

    script (MACOSX) - list of python scripts (required)
    dest_base - directory and basename for the executable
                if a directory is contained, must be the same for all targets
"""
import pkg_resources 


import setuptools 

__version__ =pkg_resources .require ("py2app")[0 ].version 



def _fixup_virtualenv (real_prefix ):
    import os 
    import sys 

    sys .real_prefix =real_prefix 





    paths =[
    os .path .join (sys .real_prefix ,"lib","python%d.%d"%(sys .version_info [:2 ]))
    ]
    hardcoded_relative_dirs =paths [:]
    plat_path =os .path .join (
    sys .real_prefix ,
    "lib",
    "python%d.%d"%(sys .version_info [:2 ]),
    "plat-%s"%sys .platform ,
    )
    if os .path .exists (plat_path ):
        paths .append (plat_path )



    for path in list (paths ):
        tk_dir =os .path .join (path ,"lib-tk")
        if os .path .exists (tk_dir ):
            paths .append (tk_dir )



    hardcoded_paths =[
    os .path .join (relative_dir ,module )
    for relative_dir in hardcoded_relative_dirs 
    for module in ("plat-darwin","plat-mac","plat-mac/lib-scriptpackages")
    ]

    for path in hardcoded_paths :
        if os .path .exists (path ):
            paths .append (path )

    sys .path .extend (paths )



def _setup_pkgresources ():
    import os 
    import plistlib 

    import pkg_resources 

    if hasattr (plistlib ,"load"):
        with open (
        os .path .join (os .path .dirname (os .getenv ("RESOURCEPATH")),"Info.plist"),"rb"
        )as fp :

            pl =plistlib .load (fp )

    else :

        pl =plistlib .readPlist (
        os .path .join (os .path .dirname (os .getenv ("RESOURCEPATH")),"Info.plist")
        )

    appname =pl .get ("CFBundleIdentifier")
    if appname is None :
        appname =pl ["CFBundleDisplayName"]
    path =os .path .expanduser ("~/Library/Caches/%s/python-eggs"%(appname ,))
    pkg_resources .set_extraction_path (path )


_setup_pkgresources ()



"""
altgraph.GraphUtil - Utility classes and functions
==================================================
"""

import random 
from collections import deque 

from altgraph import Graph ,GraphError 


def generate_random_graph (node_num ,edge_num ,self_loops =False ,multi_edges =False ):
    """
    Generates and returns a :py:class:`~altgraph.Graph.Graph` instance with
    *node_num* nodes randomly connected by *edge_num* edges.
    """
    g =Graph .Graph ()

    if not multi_edges :
        if self_loops :
            max_edges =node_num *node_num 
        else :
            max_edges =node_num *(node_num -1 )

        if edge_num >max_edges :
            raise GraphError ("inconsistent arguments to 'generate_random_graph'")

    nodes =range (node_num )

    for node in nodes :
        g .add_node (node )

    while 1 :
        head =random .choice (nodes )
        tail =random .choice (nodes )


        if head ==tail and not self_loops :
            continue 


        if g .edge_by_node (head ,tail )is not None and not multi_edges :
            continue 


        g .add_edge (head ,tail )
        if g .number_of_edges ()>=edge_num :
            break 

    return g 


def generate_scale_free_graph (steps ,growth_num ,self_loops =False ,multi_edges =False ):
    """
    Generates and returns a :py:class:`~altgraph.Graph.Graph` instance that
    will have *steps* \\* *growth_num* nodes and a scale free (powerlaw)
    connectivity. Starting with a fully connected graph with *growth_num*
    nodes at every step *growth_num* nodes are added to the graph and are
    connected to existing nodes with a probability proportional to the degree
    of these existing nodes.
    """

    graph =Graph .Graph ()


    store =[]
    for i in range (growth_num ):
        for j in range (i +1 ,growth_num ):
            store .append (i )
            store .append (j )
            graph .add_edge (i ,j )


    for node in range (growth_num ,steps *growth_num ):
        graph .add_node (node )
        while graph .out_degree (node )<growth_num :
            nbr =random .choice (store )


            if node ==nbr and not self_loops :
                continue 


            if graph .edge_by_node (node ,nbr )and not multi_edges :
                continue 

            graph .add_edge (node ,nbr )

        for nbr in graph .out_nbrs (node ):
            store .append (node )
            store .append (nbr )

    return graph 


def filter_stack (graph ,head ,filters ):
    """
    Perform a walk in a depth-first order starting
    at *head*.

    Returns (visited, removes, orphans).

    * visited: the set of visited nodes
    * removes: the list of nodes where the node
      data does not all *filters*
    * orphans: tuples of (last_good, node),
      where node is not in removes, is directly
      reachable from a node in *removes* and
      *last_good* is the closest upstream node that is not
      in *removes*.
    """

    visited ,removes ,orphans ={head },set (),set ()
    stack =deque ([(head ,head )])
    get_data =graph .node_data 
    get_edges =graph .out_edges 
    get_tail =graph .tail 

    while stack :
        last_good ,node =stack .pop ()
        data =get_data (node )
        if data is not None :
            for filtfunc in filters :
                if not filtfunc (data ):
                    removes .add (node )
                    break 
            else :
                last_good =node 
        for edge in get_edges (node ):
            tail =get_tail (edge )
            if last_good is not node :
                orphans .add ((last_good ,tail ))
            if tail not in visited :
                visited .add (tail )
                stack .append ((last_good ,tail ))

    orphans =[(lg ,tl )for (lg ,tl )in orphans if tl not in removes ]

    return visited ,removes ,orphans 



def _setup_ctypes ():
    import os 
    from ctypes .macholib import dyld 

    frameworks =os .path .join (os .environ ["RESOURCEPATH"],"..","Frameworks")
    dyld .DEFAULT_FRAMEWORK_FALLBACK .insert (0 ,frameworks )
    dyld .DEFAULT_LIBRARY_FALLBACK .insert (0 ,frameworks )


_setup_ctypes ()



"""
Recipe for pyEnchant <http://pypi.python.org/pypi/pyenchant>

PyEnchant is a python library that wraps a C library
using ctypes, hence the usual way to find the library
won't work.
"""
import os 


def check (cmd ,mf ):
    m =mf .findNode ("enchant")
    if m is None or m .filename is None :
        return None 

    if "PYENCHANT_LIBRARY_PATH"in os .environ :

        print ("WARNING: using pyEnchant without embedding")
        print ("WARNING: this is not supported at the moment")

    else :
        path =os .path .dirname (m .filename )
        if not os .path .exists (os .path .join (path ,"lib","libenchant.1.dylib")):
            print ("WARNING: using pyEnchant without embedding")
            print ("WARNING: this is not supported at the moment")



    return {"packages":["enchant"]}



def _update_path ():
    import os 
    import sys 

    resources =os .environ ["RESOURCEPATH"]
    sys .path .append (
    os .path .join (
    resources ,"lib","python%d.%d"%(sys .version_info [:2 ]),"lib-dynload"
    )
    )
    sys .path .append (
    os .path .join (resources ,"lib","python%d.%d"%(sys .version_info [:2 ]))
    )
    sys .path .append (
    os .path .join (
    resources ,
    "lib",
    "python%d.%d"%(sys .version_info [:2 ]),
    "site-packages.zip",
    )
    )


_update_path ()



import os 

from modulegraph .modulegraph import MissingModule 


def check (cmd ,mf ):
    m =mf .findNode ("PyQt6")
    if m and not isinstance (m ,MissingModule ):
        try :


            import sipconfig 

            return None 

        except ImportError :
            pass 

        try :
            import PyQt6 
            from PyQt6 .QtCore import QLibraryInfo 
        except ImportError :


            return None 

        qtdir =QLibraryInfo .path (QLibraryInfo .LibraryPath .LibrariesPath )
        if os .path .relpath (qtdir ,os .path .dirname (PyQt6 .__file__ )).startswith ("../"):




            print ("System install of Qt6")



            extra ={
            "resources":[
            ("..",[QLibraryInfo .path (QLibraryInfo .LibraryPath .PluginsPath )])
            ]
            }

        else :
            extra ={}






        try :
            mf .import_hook ("sip",m )
        except ImportError :
            mf .import_hook ("sip",m ,level =1 )

        result ={"packages":["PyQt6"]}
        result .update (extra )
        return result 

    return None 



from __future__ import print_function 

import argparse 
import os 
import sys 

from .modulegraph import ModuleGraph 


def parse_arguments ():
    parser =argparse .ArgumentParser (
    conflict_handler ="resolve",
    prog ="%s -mmodulegraph"%(os .path .basename (sys .executable )),
    )
    parser .add_argument (
    "-d",action ="count",dest ="debug",default =1 ,help ="Increase debug level"
    )
    parser .add_argument (
    "-q",action ="store_const",dest ="debug",const =0 ,help ="Clear debug level"
    )
    parser .add_argument (
    "-m",
    "--modules",
    action ="store_true",
    dest ="domods",
    default =False ,
    help ="arguments are module names, not script files",
    )
    parser .add_argument (
    "-x",
    metavar ="NAME",
    action ="append",
    dest ="excludes",
    default =[],
    help ="Add NAME to the excludes list",
    )
    parser .add_argument (
    "-p",
    action ="append",
    metavar ="PATH",
    dest ="addpath",
    default =[],
    help ="Add PATH to the module search path",
    )
    parser .add_argument (
    "-g",
    "--dot",
    action ="store_const",
    dest ="output",
    const ="dot",
    help ="Output a .dot graph",
    )
    parser .add_argument (
    "-h",
    "--html",
    action ="store_const",
    dest ="output",
    const ="html",
    help ="Output a HTML file",
    )
    parser .add_argument (
    "scripts",metavar ="SCRIPT",nargs ="+",help ="scripts to analyse"
    )

    opts =parser .parse_args ()
    return opts 


def create_graph (scripts ,domods ,debuglevel ,excludes ,path_extras ):

    path =sys .path [:]

    if domods :
        del path [0 ]
    else :
        path [0 ]=os .path .dirname (scripts [0 ])

    path =path_extras +path 
    if debuglevel >1 :
        print ("path:",file =sys .stderr )
        for item in path :
            print ("   ",repr (item ),file =sys .stderr )


    mf =ModuleGraph (path ,excludes =excludes ,debug =debuglevel )
    for arg in scripts :
        if domods :
            if arg [-2 :]==".*":
                mf .import_hook (arg [:-2 ],None ,["*"])
            else :
                mf .import_hook (arg )
        else :
            mf .run_script (arg )
    return mf 


def output_graph (output_format ,mf ):
    if output_format =="dot":
        mf .graphreport ()
    elif output_format =="html":
        mf .create_xref ()
    else :
        mf .report ()


def main ():
    opts =parse_arguments ()
    mf =create_graph (
    opts .scripts ,opts .domods ,opts .debug ,opts .excludes ,opts .addpath 
    )
    output_graph (opts .output ,mf )


if __name__ =="__main__":
    try :
        main ()
    except KeyboardInterrupt :
        print ("\n[interrupt]")



def _import_encodings ():
    import encodings 
    import os 
    import pkgutil 
    import sys 

    del sys .path [:2 ]
    import encodings .aliases 

    encodings .__path__ =pkgutil .extend_path (encodings .__path__ ,encodings .__name__ )

    import encodings .mac_roman 

    encodings .aliases .__file__ =os .path .join (
    os .path .dirname (encodings .mac_roman .__file__ ),
    "aliases.py"+encodings .mac_roman .__file__ [:-1 ],
    )

    if sys .version_info [:2 ]<(3 ,4 ):
        from imp import reload 
    else :
        from importlib import reload 

    reload (encodings .aliases )
    reload (encodings )


_import_encodings ()



PYDANTIC_IMPORTS =[
"abc",
"collections",
"collections.abc",
"colorsys",
"configparser",
"contextlib",
"copy",
"dataclasses",
"datetime",
"decimal",
"enum",
"fractions",
"functools",
"ipaddress",
"itertools",
"json",
"math",
"os",
"pathlib",
"pickle",
"re",
"sys",
"types",
"typing",
"typing_extensions",
"uuid",
"warnings",
"weakref",
]


def check (cmd ,mf ):
    m =mf .findNode ("pydantic")
    if m is None or m .filename is None :
        return None 



    return {"packages":["pydantic"],"includes":PYDANTIC_IMPORTS }



"""
Generic framework path manipulation
"""

import re 

__all__ =["framework_info"]

_STRICT_FRAMEWORK_RE =re .compile (
r"""(?x)
(?P<location>^.*)(?:^|/)
(?P<name>
    (?P<shortname>[-_A-Za-z0-9]+).framework/
    (?:Versions/(?P<version>[^/]+)/)?
    (?P=shortname)
    (?:_(?P<suffix>[^_]+))?
)$
"""
)


def framework_info (filename ):
    """
    A framework name can take one of the following four forms:
        Location/Name.framework/Versions/SomeVersion/Name_Suffix
        Location/Name.framework/Versions/SomeVersion/Name
        Location/Name.framework/Name_Suffix
        Location/Name.framework/Name

    returns None if not found, or a mapping equivalent to:
        dict(
            location='Location',
            name='Name.framework/Versions/SomeVersion/Name_Suffix',
            shortname='Name',
            version='SomeVersion',
            suffix='Suffix',
        )

    Note that SomeVersion and Suffix are optional and may be None
    if not present
    """
    is_framework =_STRICT_FRAMEWORK_RE .match (filename )
    if not is_framework :
        return None 
    return is_framework .groupdict ()



def _disable_linecache ():
    import linecache 

    def fake_getline (*args ,**kwargs ):
        return ""

    linecache .orig_getline =linecache .getline 
    linecache .getline =fake_getline 


_disable_linecache ()



"""
altgraph.Dot - Interface to the dot language
============================================

The :py:mod:`~altgraph.Dot` module provides a simple interface to the
file format used in the
`graphviz <http://www.research.att.com/sw/tools/graphviz/>`_
program. The module is intended to offload the most tedious part of the process
(the **dot** file generation) while transparently exposing most of its
features.

To display the graphs or to generate image files the
`graphviz <http://www.research.att.com/sw/tools/graphviz/>`_
package needs to be installed on the system, moreover the :command:`dot` and
:command:`dotty` programs must be accesible in the program path so that they
can be ran from processes spawned within the module.

Example usage
-------------

Here is a typical usage::

    from altgraph import Graph, Dot

    # create a graph
    edges = [ (1,2), (1,3), (3,4), (3,5), (4,5), (5,4) ]
    graph = Graph.Graph(edges)

    # create a dot representation of the graph
    dot = Dot.Dot(graph)

    # display the graph
    dot.display()

    # save the dot representation into the mydot.dot file
    dot.save_dot(file_name='mydot.dot')

    # save dot file as gif image into the graph.gif file
    dot.save_img(file_name='graph', file_type='gif')

Directed graph and non-directed graph
-------------------------------------

Dot class can use for both directed graph and non-directed graph
by passing ``graphtype`` parameter.

Example::

    # create directed graph(default)
    dot = Dot.Dot(graph, graphtype="digraph")

    # create non-directed graph
    dot = Dot.Dot(graph, graphtype="graph")

Customizing the output
----------------------

The graph drawing process may be customized by passing
valid :command:`dot` parameters for the nodes and edges. For a list of all
parameters see the `graphviz <http://www.research.att.com/sw/tools/graphviz/>`_
documentation.

Example::

    # customizing the way the overall graph is drawn
    dot.style(size='10,10', rankdir='RL', page='5, 5' , ranksep=0.75)

    # customizing node drawing
    dot.node_style(1, label='BASE_NODE',shape='box', color='blue' )
    dot.node_style(2, style='filled', fillcolor='red')

    # customizing edge drawing
    dot.edge_style(1, 2, style='dotted')
    dot.edge_style(3, 5, arrowhead='dot', label='binds', labelangle='90')
    dot.edge_style(4, 5, arrowsize=2, style='bold')


.. note::

   dotty (invoked via :py:func:`~altgraph.Dot.display`) may not be able to
   display all graphics styles. To verify the output save it to an image file
   and look at it that way.

Valid attributes
----------------

    - dot styles, passed via the :py:meth:`Dot.style` method::

        rankdir = 'LR'   (draws the graph horizontally, left to right)
        ranksep = number (rank separation in inches)

    - node attributes, passed via the :py:meth:`Dot.node_style` method::

        style = 'filled' | 'invisible' | 'diagonals' | 'rounded'
        shape = 'box' | 'ellipse' | 'circle' | 'point' | 'triangle'

    - edge attributes, passed via the :py:meth:`Dot.edge_style` method::

        style     = 'dashed' | 'dotted' | 'solid' | 'invis' | 'bold'
        arrowhead = 'box' | 'crow' | 'diamond' | 'dot' | 'inv' | 'none'
            | 'tee' | 'vee'
        weight    = number (the larger the number the closer the nodes will be)

    - valid `graphviz colors
        <http://www.research.att.com/~erg/graphviz/info/colors.html>`_

    - for more details on how to control the graph drawing process see the
      `graphviz reference
        <http://www.research.att.com/sw/tools/graphviz/refs.html>`_.
"""
import os 
import warnings 

from altgraph import GraphError 


class Dot (object ):
    """
    A  class providing a **graphviz** (dot language) representation
    allowing a fine grained control over how the graph is being
    displayed.

    If the :command:`dot` and :command:`dotty` programs are not in the current
    system path their location needs to be specified in the contructor.
    """

    def __init__ (
    self ,
    graph =None ,
    nodes =None ,
    edgefn =None ,
    nodevisitor =None ,
    edgevisitor =None ,
    name ="G",
    dot ="dot",
    dotty ="dotty",
    neato ="neato",
    graphtype ="digraph",
    ):
        """
        Initialization.
        """
        self .name ,self .attr =name ,{}

        assert graphtype in ["graph","digraph"]
        self .type =graphtype 

        self .temp_dot ="tmp_dot.dot"
        self .temp_neo ="tmp_neo.dot"

        self .dot ,self .dotty ,self .neato =dot ,dotty ,neato 



        self .nodes ,self .edges ={},{}

        if graph is not None and nodes is None :
            nodes =graph 
        if graph is not None and edgefn is None :

            def edgefn (node ,graph =graph ):
                return graph .out_nbrs (node )

        if nodes is None :
            nodes =()

        seen =set ()
        for node in nodes :
            if nodevisitor is None :
                style ={}
            else :
                style =nodevisitor (node )
            if style is not None :
                self .nodes [node ]={}
                self .node_style (node ,**style )
                seen .add (node )
        if edgefn is not None :
            for head in seen :
                for tail in (n for n in edgefn (head )if n in seen ):
                    if edgevisitor is None :
                        edgestyle ={}
                    else :
                        edgestyle =edgevisitor (head ,tail )
                    if edgestyle is not None :
                        if head not in self .edges :
                            self .edges [head ]={}
                        self .edges [head ][tail ]={}
                        self .edge_style (head ,tail ,**edgestyle )

    def style (self ,**attr ):
        """
        Changes the overall style
        """
        self .attr =attr 

    def display (self ,mode ="dot"):
        """
        Displays the current graph via dotty
        """

        if mode =="neato":
            self .save_dot (self .temp_neo )
            neato_cmd ="%s -o %s %s"%(self .neato ,self .temp_dot ,self .temp_neo )
            os .system (neato_cmd )
        else :
            self .save_dot (self .temp_dot )

        plot_cmd ="%s %s"%(self .dotty ,self .temp_dot )
        os .system (plot_cmd )

    def node_style (self ,node ,**kwargs ):
        """
        Modifies a node style to the dot representation.
        """
        if node not in self .edges :
            self .edges [node ]={}
        self .nodes [node ]=kwargs 

    def all_node_style (self ,**kwargs ):
        """
        Modifies all node styles
        """
        for node in self .nodes :
            self .node_style (node ,**kwargs )

    def edge_style (self ,head ,tail ,**kwargs ):
        """
        Modifies an edge style to the dot representation.
        """
        if tail not in self .nodes :
            raise GraphError ("invalid node %s"%(tail ,))

        try :
            if tail not in self .edges [head ]:
                self .edges [head ][tail ]={}
            self .edges [head ][tail ]=kwargs 
        except KeyError :
            raise GraphError ("invalid edge  %s -> %s "%(head ,tail ))

    def iterdot (self ):

        if self .type =="digraph":
            yield "digraph %s {\n"%(self .name ,)
        elif self .type =="graph":
            yield "graph %s {\n"%(self .name ,)

        else :
            raise GraphError ("unsupported graphtype %s"%(self .type ,))


        for attr_name ,attr_value in sorted (self .attr .items ()):
            yield '%s="%s";'%(attr_name ,attr_value )
        yield "\n"


        cpatt ='%s="%s",'
        epatt ="];\n"


        for node_name ,node_attr in sorted (self .nodes .items ()):
            yield '\t"%s" ['%(node_name ,)
            for attr_name ,attr_value in sorted (node_attr .items ()):
                yield cpatt %(attr_name ,attr_value )
            yield epatt 


        for head in sorted (self .edges ):
            for tail in sorted (self .edges [head ]):
                if self .type =="digraph":
                    yield '\t"%s" -> "%s" ['%(head ,tail )
                else :
                    yield '\t"%s" -- "%s" ['%(head ,tail )
                for attr_name ,attr_value in sorted (self .edges [head ][tail ].items ()):
                    yield cpatt %(attr_name ,attr_value )
                yield epatt 


        yield "}\n"

    def __iter__ (self ):
        return self .iterdot ()

    def save_dot (self ,file_name =None ):
        """
        Saves the current graph representation into a file
        """

        if not file_name :
            warnings .warn (DeprecationWarning ,"always pass a file_name",stacklevel =2 )
            file_name =self .temp_dot 

        with open (file_name ,"w")as fp :
            for chunk in self .iterdot ():
                fp .write (chunk )

    def save_img (self ,file_name =None ,file_type ="gif",mode ="dot"):
        """
        Saves the dot file as an image file
        """

        if not file_name :
            warnings .warn (DeprecationWarning ,"always pass a file_name",stacklevel =2 )
            file_name ="out"

        if mode =="neato":
            self .save_dot (self .temp_neo )
            neato_cmd ="%s -o %s %s"%(self .neato ,self .temp_dot ,self .temp_neo )
            os .system (neato_cmd )
            plot_cmd =self .dot 
        else :
            self .save_dot (self .temp_dot )
            plot_cmd =self .dot 

        file_name ="%s.%s"%(file_name ,file_type )
        create_cmd ="%s -T%s %s -o %s"%(
        plot_cmd ,
        file_type ,
        self .temp_dot ,
        file_name ,
        )
        os .system (create_cmd )



import sys
import os
from PySide6.QtWidgets import QApplication, QWidget, QLabel, QPushButton, QCheckBox, QComboBox, QListWidget, QVBoxLayout, QHBoxLayout, QFileDialog, QMessageBox, QStyleFactory, QInputDialog, QDialog
from PySide6.QtCore import Qt
from PySide6.QtGui import QPalette, QColor, QFont
import numpy as np
import h5py
from neo.rawio import AxonRawIO
import pyabf
from heka_reader import Bundle


class SDCombineDatasets(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("SD Combine Datasets/Files")
        self.resize(500, 700)

        # Create widgets
        self.app_name_label = QLabel("SD Combine Datasets/Files")
        self.app_name_label.setAlignment(Qt.AlignCenter)
        self.app_name_label.setFont(QFont("Arial", 16, QFont.Bold))
        self.email_label = QLabel("shankar.dutt@anu.edu.au")
        self.email_label.setAlignment(Qt.AlignCenter)
        self.select_folder_button = QPushButton("Select Folder")
        self.include_subfolders_checkbox = QCheckBox("Include Subfolders")
        self.extension_dropdown = QComboBox()
        self.extension_dropdown.addItems([".dataset.npz", ".MLdataset.npz", ".abf", ".h5", ".dat", ".event_data.npz", ".event_fitting.npz"])
        self.file_list = QListWidget()
        self.file_list.setSelectionMode(QListWidget.SelectionMode.MultiSelection)
        self.select_all_checkbox = QCheckBox("Select All")
        self.same_duration_checkbox = QCheckBox("Files have the same duration")
        self.same_duration_checkbox.setChecked(True)  # Set checked by default
        self.folder_path_label = QLabel()
        self.folder_path_label.setWordWrap(True)
        self.combine_datasets_button = QPushButton("Combine Datasets/Files")

        # Create layouts
        main_layout = QVBoxLayout()
        main_layout.addWidget(self.app_name_label)
        main_layout.addWidget(self.email_label)
        main_layout.addWidget(self.select_folder_button)
        main_layout.addWidget(self.include_subfolders_checkbox)
        main_layout.addWidget(self.extension_dropdown)
        main_layout.addWidget(self.file_list)
        main_layout.addWidget(self.select_all_checkbox)
        main_layout.addWidget(self.same_duration_checkbox)
        main_layout.addWidget(self.folder_path_label)
        main_layout.addWidget(self.combine_datasets_button)

        self.setLayout(main_layout)

        # Connect signals and slots
        self.select_folder_button.clicked.connect(self.select_folder)
        self.select_all_checkbox.stateChanged.connect(self.select_all)
        self.combine_datasets_button.clicked.connect(self.combine_datasets)

    def select_folder(self):
        folder_path = QFileDialog.getExistingDirectory(self, "Select Folder")
        if folder_path:
            self.file_list.clear()
            self.folder_path_label.setText(f"Selected Folder: {folder_path}")
            extension = self.extension_dropdown.currentText()
            if self.include_subfolders_checkbox.isChecked():
                for root, dirs, files in os.walk(folder_path):
                    for file in files:
                        if file.endswith(extension):
                            file_path = os.path.relpath(os.path.join(root, file), folder_path)
                            self.file_list.addItem(file_path)
            else:
                for file in os.listdir(folder_path):
                    if file.endswith(extension):
                        self.file_list.addItem(file)

    def select_all(self, state):
        is_checked = self.select_all_checkbox.isChecked()
        for index in range(self.file_list.count()):
            item = self.file_list.item(index)
            item.setSelected(is_checked)

    def read_abf_file(self, file_path):
        try:
            # Create an instance of AxonRawIO
            raw_io = AxonRawIO(filename=file_path)
            
            # Parse the header information
            raw_io.parse_header()
            
            # Define the channel index you're interested in
            channel_index = 0
            
            # Read the signal size for the given block and segment without specifying channel_indexes
            signal_size = raw_io.get_signal_size(block_index=0, seg_index=0)
            
            # Now, read the analog signal data
            # Here, we specify channel_indexes when reading the chunk to ensure we only get data for the desired channel
            data = raw_io.get_analogsignal_chunk(block_index=0, seg_index=0, i_start=0, i_stop=signal_size, channel_indexes=[channel_index])
            
            # Convert the chunk to a physical quantity
            data = raw_io.rescale_signal_raw_to_float(data, dtype='float64', channel_indexes=[channel_index]).flatten()
            
            # Get the sampling rate for the specified channel
            sampling_rate = raw_io.get_signal_sampling_rate()
            return data, sampling_rate
        except:
            abf = pyabf.ABF(file_path)
            data = abf.sweepY
            sampling_rate = abf.dataRate
            return data, sampling_rate

    def get_heka_trace_info(self, file_path):
        """
        Get information about available traces in a HEKA file.
        Returns a list of (trace_idx, label, unit) tuples.
        """
        try:
            bundle = Bundle(file_path)
            pul = bundle.pul

            trace_info = []
            # Get trace info from the first available sweep
            for group in pul:
                for series in group:
                    for sweep in series:
                        for trace_idx, trace in enumerate(sweep):
                            label = trace.Label if trace.Label else f"Trace {trace_idx}"
                            unit = trace.YUnit if trace.YUnit else "?"
                            trace_info.append((trace_idx, label, unit))
                        if trace_info:
                            return trace_info
            return trace_info
        except Exception as e:
            print(f"Error getting trace info: {e}")
            return []

    def read_heka_file(self, file_path, selected_trace_idx=0):
        """
        Read HEKA .dat file and return all series combined as a continuous trace.
        Uses heka_reader (campagnola/heka_reader from GitHub).
        Returns combined data (converted to nA for current) and sampling rate.

        Args:
            file_path: Path to the HEKA .dat file
            selected_trace_idx: Index of the trace to extract (0=Current, 1=Voltage typically)
        """
        try:
            # Load the HEKA bundle file
            bundle = Bundle(file_path)

            # Collect all data from all groups, series, sweeps for the selected trace
            all_data = []
            sampling_rate = None
            y_unit = None

            # Navigate the hierarchy: groups > series > sweeps > traces
            pul = bundle.pul

            for group_idx, group in enumerate(pul):
                for series_idx, series in enumerate(group):
                    for sweep_idx, sweep in enumerate(series):
                        # Only get the selected trace index
                        if selected_trace_idx < len(sweep):
                            trace = sweep[selected_trace_idx]
                            try:
                                # Get data for this trace
                                data = bundle.data[group_idx, series_idx, sweep_idx, selected_trace_idx]

                                if len(data) > 0:
                                    all_data.append(data)

                                    # Get sampling rate and unit from the first trace with data
                                    if sampling_rate is None and trace.XInterval > 0:
                                        sampling_rate = 1.0 / trace.XInterval  # XInterval is in seconds
                                    if y_unit is None:
                                        y_unit = trace.YUnit
                            except Exception as e:
                                print(f"Warning: Could not read trace at [{group_idx}][{series_idx}][{sweep_idx}][{selected_trace_idx}]: {e}")
                                continue

            if all_data and sampling_rate is not None:
                # Concatenate all traces into one continuous trace
                combined_data = np.concatenate(all_data)

                # Convert to nA based on the original unit
                combined_data = self.convert_to_nA(combined_data, y_unit)

                return combined_data, sampling_rate
            else:
                raise ValueError("No valid data could be read from the file")

        except Exception as e:
            raise Exception(f"Error reading HEKA file: {e}")

    def convert_to_nA(self, data, y_unit):
        """
        Convert current data to nanoamperes (nA).

        Args:
            data: numpy array of current values
            y_unit: unit string from HEKA file (e.g., 'pA', 'nA', 'A', 'µA')

        Returns:
            data converted to nA
        """
        if y_unit is None:
            print("Warning: No unit information found, assuming data is already in nA")
            return data

        y_unit = y_unit.strip().lower()

        if y_unit in ['pa', 'pa']:  # picoamperes
            return data / 1000.0  # pA to nA
        elif y_unit in ['na', 'na']:  # nanoamperes
            return data  # Already in nA
        elif y_unit in ['a', 'amp', 'amps']:  # amperes
            return data * 1e9  # A to nA
        elif y_unit in ['µa', 'ua', 'μa', 'microamp']:  # microamperes
            return data * 1000.0  # µA to nA
        elif y_unit in ['ma']:  # milliamperes
            return data * 1e6  # mA to nA
        elif y_unit in ['mv', 'v']:  # Voltage - don't convert
            print(f"Note: Data is in voltage units ({y_unit}), not converting")
            return data
        else:
            print(f"Warning: Unknown unit '{y_unit}', assuming data is already in nA")
            return data

    def convert_heka_to_h5(self, selected_files, folder_path):
        """
        Convert HEKA .dat files to .h5 format.
        Combines all series within each file and appends data from multiple files.
        """
        if not selected_files:
            return

        # Get trace info from the first file to let user select which trace to extract
        first_file_path = os.path.join(folder_path, selected_files[0])
        trace_info = self.get_heka_trace_info(first_file_path)

        if not trace_info:
            QMessageBox.critical(self, "Error", "Could not read trace information from the HEKA file.")
            return

        # Build selection list for dialog
        trace_options = []
        for trace_idx, label, unit in trace_info:
            trace_options.append(f"{trace_idx}: {label} ({unit})")

        # Ask user which trace to extract
        selected_option, ok = QInputDialog.getItem(
            self,
            "Select Data Channel",
            "Select which channel to extract:\n(Typically: 0=Current, 1=Voltage)",
            trace_options,
            0,  # Default to first item (usually current)
            False  # Not editable
        )

        if not ok:
            return

        # Parse selected trace index
        selected_trace_idx = int(selected_option.split(":")[0])
        selected_label = trace_info[selected_trace_idx][1]

        combined_data = []
        sampling_rate = None

        for file_path in selected_files:
            full_path = os.path.join(folder_path, file_path)
            try:
                data, file_sampling_rate = self.read_heka_file(full_path, selected_trace_idx)
                combined_data.append(data)

                if sampling_rate is None:
                    sampling_rate = file_sampling_rate
                elif sampling_rate != file_sampling_rate:
                    QMessageBox.warning(
                        self,
                        "Warning",
                        f"File {file_path} has different sampling rate ({file_sampling_rate} Hz vs {sampling_rate} Hz). "
                        "Proceeding with combination, but please be cautious with the results."
                    )
            except Exception as e:
                QMessageBox.warning(self, "Warning", f"Could not read file {file_path}: {e}")
                continue

        if not combined_data:
            QMessageBox.critical(self, "Error", "No valid data could be read from the selected files.")
            return

        # Combine all data into one continuous trace
        combined_data = np.concatenate(combined_data)

        # Save as .h5 file
        save_path, _ = QFileDialog.getSaveFileName(
            self,
            "Save Converted HEKA Data",
            "",
            "HDF5 Files (*.h5)"
        )

        if save_path:
            if not save_path.endswith('.h5'):
                save_path += '.h5'

            with h5py.File(save_path, 'w') as f:
                f.create_dataset('selected_data', data=combined_data)
                f.attrs['sampling_rate'] = sampling_rate

            # Calculate duration for info message
            duration = len(combined_data) / sampling_rate
            QMessageBox.information(
                self,
                "Success",
                f"HEKA data converted successfully and saved as .h5 file.\n\n"
                f"Channel: {selected_label}\n"
                f"Total data points: {len(combined_data):,}\n"
                f"Sampling rate: {sampling_rate:,.0f} Hz\n"
                f"Duration: {duration:.2f} seconds"
            )

    def combine_datasets(self):
        selected_files = [self.file_list.item(i).text() for i in range(self.file_list.count()) if self.file_list.item(i).isSelected()]
        if selected_files:
            if self.select_all_checkbox.isChecked():
                selected_files.sort()  # Sort files by name if all files are selected
            else:
                selected_indexes = self.file_list.selectedIndexes()
                selected_files = [self.file_list.item(index.row()).text() for index in selected_indexes]  # Preserve user's selection order

            extension = self.extension_dropdown.currentText()
            folder_path = self.folder_path_label.text().replace("Selected Folder: ", "")

            if extension in [".dataset.npz", ".MLdataset.npz"]:
                self.combine_npz_datasets(selected_files, folder_path, extension)
            elif extension in [".abf", ".h5"]:
                self.combine_abf_h5_datasets(selected_files, folder_path, extension)
            elif extension == ".dat":
                self.convert_heka_to_h5(selected_files, folder_path)
            elif extension == ".event_data.npz":
                self.combine_event_data_npz(selected_files, folder_path)
            elif extension == ".event_fitting.npz":
                self.combine_event_fitting_npz(selected_files, folder_path)
        else:
            QMessageBox.warning(self, "Warning", "No files selected.")


    def combine_event_fitting_npz(self, selected_files, folder_path):
        combined_npz_dict = {}
        all_event_analyses = []
        event_counter = 0

        file_lengths = []
        if self.same_duration_checkbox.isChecked():
            file_length, ok = QInputDialog.getInt(self, "File Duration", "Enter the duration of each file in seconds:")
            if ok:
                file_lengths = [file_length] * len(selected_files)
            else:
                QMessageBox.warning(self, "Warning", "File duration not provided. Skipping combination.")
                return
        else:
            for file_path in selected_files:
                file_length, ok = QInputDialog.getInt(self, "File Duration", f"Enter the duration of file '{file_path}' in seconds:")
                if ok:
                    file_lengths.append(file_length)
                else:
                    QMessageBox.warning(self, "Warning", "File duration not provided. Skipping file.")
                    return

        cumulative_time = 0
        for file_path, file_length in zip(selected_files, file_lengths):
            full_path = os.path.join(folder_path, file_path)
            with np.load(full_path, allow_pickle=True) as data:
                for key, value in data.items():
                    if key.startswith('EVENT_DATA_'):
                        event_num = int(key.split('_')[2])
                        new_key = f'EVENT_DATA_{event_counter}'
                        if key.endswith('_part_2'):  # This is the event_time_start_end array
                            value = value + cumulative_time
                        combined_npz_dict[f'{new_key}{key[key.rfind("_"):]}'] = value
                    elif key.startswith('SEGMENT_INFO_'):
                        event_num = int(key.split('_')[2])
                        new_key = f'SEGMENT_INFO_{event_counter}'
                        combined_npz_dict[f'{new_key}{key[key.rfind("_"):]}'] = value
                    elif key.startswith('EVENT_ANALYSIS_'):
                        event_num = int(key.split('_')[2])
                        new_key = f'EVENT_ANALYSIS_{event_counter}'
                        adjusted_value = value.copy()
                        adjusted_value[-1] += cumulative_time  # Adjust the last element (start time)
                        combined_npz_dict[new_key] = adjusted_value
                        all_event_analyses.append(adjusted_value)
                    event_counter += 1
            cumulative_time += file_length

        if combined_npz_dict:
            save_path, _ = QFileDialog.getSaveFileName(self, "Save Combined Event Fitting Data", "", "NumPy Files (*.event_fitting.npz)")
            if save_path:
                if not save_path.endswith('.event_fitting.npz'):
                    save_path += '.event_fitting.npz'
                np.savez_compressed(save_path, **combined_npz_dict)
                
                # Save the combined event analyses as a separate file
                analysis_save_path = save_path.replace('.event_fitting.npz', '.event_analysis.npz')
                np.savez_compressed(analysis_save_path, event_analyses=np.array(all_event_analyses, dtype=object))
                
                QMessageBox.information(self, "Success", "Event fitting data files combined successfully.")
        else:
            QMessageBox.warning(self, "Warning", "No valid event fitting data files to combine.")

    def combine_event_data_npz(self, selected_files, folder_path):
        combined_events = []
        sampling_rate = None
        event_id_offset = 0
        cumulative_time = 0

        file_lengths = []
        if self.same_duration_checkbox.isChecked():
            file_length, ok = QInputDialog.getInt(self, "File Duration", "Enter the duration of each file in seconds:")
            if ok:
                file_lengths = [file_length] * len(selected_files)
            else:
                QMessageBox.warning(self, "Warning", "File duration not provided. Skipping combination.")
                return
        else:
            for file_path in selected_files:
                file_length, ok = QInputDialog.getInt(self, "File Duration", f"Enter the duration of file '{file_path}' in seconds:")
                if ok:
                    file_lengths.append(file_length)
                else:
                    QMessageBox.warning(self, "Warning", "File duration not provided. Skipping file.")
                    return

        for file_path, file_length in zip(selected_files, file_lengths):
            full_path = os.path.join(folder_path, file_path)
            with np.load(full_path, allow_pickle=True) as data:
                if sampling_rate is None:
                    sampling_rate = data['sampling_rate']
                elif sampling_rate != data['sampling_rate']:
                    QMessageBox.warning(self, "Warning", f"File {file_path} has a different sampling rate. Skipping.")
                    continue

                events = data['events']
                for event in events:
                    adjusted_event = event.copy()
                    adjusted_event['event_id'] += event_id_offset
                    adjusted_event['start_time'] += cumulative_time
                    adjusted_event['end_time'] += cumulative_time
                    combined_events.append(adjusted_event)

            event_id_offset = len(combined_events)
            cumulative_time += file_length

        if combined_events:
            save_path, _ = QFileDialog.getSaveFileName(self, "Save Combined Event Data", "", "NumPy Files (*.event_data.npz)")
            if save_path:
                if not save_path.endswith('.event_data.npz'):
                    save_path += '.event_data.npz'
                np.savez_compressed(save_path, sampling_rate=sampling_rate, events=combined_events)
                QMessageBox.information(self, "Success", "Event data files combined successfully.")
        else:
            QMessageBox.warning(self, "Warning", "No valid event data files to combine.")

    # def combine_npz_datasets(self, selected_files, folder_path, extension):
    #     file_lengths = []
    #     if self.same_duration_checkbox.isChecked():
    #         file_length, ok = QInputDialog.getInt(self, "File Duration", "Enter the duration of each file in seconds:")
    #         if ok:
    #             file_lengths = [file_length] * len(selected_files)
    #         else:
    #             QMessageBox.warning(self, "Warning", "File duration not provided. Skipping combination.")
    #             return
    #     else:
    #         for file_path in selected_files:
    #             file_length, ok = QInputDialog.getInt(self, "File Duration", f"Enter the duration of file '{file_path}' in seconds:")
    #             if ok:
    #                 file_lengths.append(file_length)
    #             else:
    #                 QMessageBox.warning(self, "Warning", "File duration not provided. Skipping file.")
    #                 return

    #     combined_data = None
    #     settings_file = None
    #     cumulative_time = 0
    #     for file_path, file_length in zip(selected_files, file_lengths):
    #         full_path = os.path.join(folder_path, file_path)
    #         with np.load(full_path) as data:
    #             if combined_data is None:
    #                 combined_data = data['X']
    #             else:
    #                 try:
    #                     data_x = data['X']
    #                     data_x[:, 8] += cumulative_time  # Add cumulative time to event_time
    #                     combined_data = np.concatenate((combined_data, data_x))
    #                 except:
    #                     print(f"Cannot save file: {file_path}")
    #             if settings_file is None:
    #                 settings_file = data['settings']
    #         cumulative_time += file_length

    #     save_path, _ = QFileDialog.getSaveFileName(self, "Save Combined Dataset", "", f"NumPy Files (*{extension})")
    #     if save_path:
    #         if not save_path.endswith(extension):
    #             save_path += extension
    #         np.savez(save_path, X=combined_data, settings=settings_file)
    #         QMessageBox.information(self, "Success", "Datasets/Files combined successfully.")

    def combine_npz_datasets(self, selected_files, folder_path, extension):
        file_lengths = []
        if self.same_duration_checkbox.isChecked():
            file_length, ok = QInputDialog.getInt(self, "File Duration", "Enter the duration of each file in seconds:")
            if ok:
                file_lengths = [file_length] * len(selected_files)
            else:
                QMessageBox.warning(self, "Warning", "File duration not provided. Skipping combination.")
                return
        else:
            for file_path in selected_files:
                file_length, ok = QInputDialog.getInt(self, "File Duration", f"Enter the duration of file '{file_path}' in seconds:")
                if ok:
                    file_lengths.append(file_length)
                else:
                    QMessageBox.warning(self, "Warning", "File duration not provided. Skipping file.")
                    return

        combined_data = None
        settings_file = None
        cumulative_time = 0
        skipped_files = []
        has_time_column = None  # Track if we found column 8 (time) in the first file
        num_columns = None  # Track the number of columns for consistency check
        
        for file_path, file_length in zip(selected_files, file_lengths):
            full_path = os.path.join(folder_path, file_path)
            try:
                with np.load(full_path) as data:
                    if combined_data is None:
                        # First file sets the baseline
                        combined_data = data['X']
                        num_columns = combined_data.shape[1]
                        has_time_column = num_columns > 8
                        
                        if not has_time_column:
                            QMessageBox.warning(self, "Warning", 
                                f"First file {file_path} has {num_columns} columns, fewer than expected (no time column). " +
                                "Will proceed with combination but time adjustments will be skipped.")
                        
                        # Try to get settings, but don't fail if not found
                        try:
                            settings_file = data['settings']
                        except KeyError:
                            QMessageBox.warning(self, "Warning", 
                                f"Settings not found in first file {file_path}. Will proceed without settings.")
                    else:
                        try:
                            data_x = data['X']
                            current_columns = data_x.shape[1]
                            
                            # Check if current file has same number of columns
                            if current_columns != num_columns:
                                raise ValueError(
                                    f"File has {current_columns} columns while first file had {num_columns} columns")
                            
                            # Only adjust time if we have the time column
                            if has_time_column:
                                data_x[:, 8] += cumulative_time  # Add cumulative time to event_time
                            
                            combined_data = np.concatenate((combined_data, data_x))
                        except Exception as e:
                            skipped_files.append((file_path, str(e)))
                            QMessageBox.warning(self, "Warning", 
                                f"Error processing file {file_path}: {str(e)}. Skipping this file.")
                            continue
                cumulative_time += file_length
            except Exception as e:
                skipped_files.append((file_path, str(e)))
                QMessageBox.warning(self, "Warning", 
                    f"Could not open file {file_path}: {str(e)}. Skipping this file.")
                continue

        if combined_data is not None:
            save_path, _ = QFileDialog.getSaveFileName(self, "Save Combined Dataset", "", f"NumPy Files (*{extension})")
            if save_path:
                if not save_path.endswith(extension):
                    save_path += extension
                
                # Save with or without settings based on what we found
                if settings_file is not None:
                    np.savez(save_path, X=combined_data, settings=settings_file)
                else:
                    np.savez(save_path, X=combined_data)
                
                # Show summary message
                success_msg = f"Datasets/Files combined successfully with {combined_data.shape[1]} columns."
                if not has_time_column:
                    success_msg += "\nNote: Time column adjustments were skipped due to insufficient columns."
                if skipped_files:
                    success_msg += "\n\nSkipped files:"
                    for file_path, error in skipped_files:
                        success_msg += f"\n- {file_path}: {error}"
                
                QMessageBox.information(self, "Success", success_msg)
        else:
            QMessageBox.critical(self, "Error", "No valid data could be combined. Please check the input files.")

    def combine_abf_h5_datasets(self, selected_files, folder_path, extension):
        combined_data = []
        sampling_rate = None

        for file_path in selected_files:
            full_path = os.path.join(folder_path, file_path)
            if extension == ".abf":
                data, file_sampling_rate = self.read_abf_file(full_path)
            elif extension == ".h5":
                with h5py.File(full_path, 'r') as f:
                    data = f['selected_data'][:]
                    file_sampling_rate = f.attrs['sampling_rate']
            
            combined_data.append(data)
            
            if sampling_rate is None:
                sampling_rate = file_sampling_rate
            elif sampling_rate != file_sampling_rate:
                QMessageBox.warning(self, "Warning", "Files have different sampling rates. Proceeding with combination, but please be cautious with the results.")

        # Combine the data
        combined_data = np.concatenate(combined_data)

        # Save as .h5 file
        save_path, _ = QFileDialog.getSaveFileName(self, "Save Combined Dataset", "", "HDF5 Files (*.h5)")
        if save_path:
            if not save_path.endswith('.h5'):
                save_path += '.h5'
            with h5py.File(save_path, 'w') as f:
                f.create_dataset('selected_data', data=combined_data)
                f.attrs['sampling_rate'] = sampling_rate
            QMessageBox.information(self, "Success", "Datasets combined successfully and saved as .h5 file.")

if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setStyle(QStyleFactory.create('Fusion'))  # Use Fusion or other available styles

    # Customize the palette for a darker, more modern look
    palette = QPalette()
    palette.setColor(QPalette.ColorRole.Window, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.WindowText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Base, QColor(25, 25, 25))
    palette.setColor(QPalette.ColorRole.AlternateBase, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.ToolTipBase, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.ToolTipText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Text, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.Button, QColor(53, 53, 53))
    palette.setColor(QPalette.ColorRole.ButtonText, Qt.GlobalColor.white)
    palette.setColor(QPalette.ColorRole.BrightText, Qt.GlobalColor.red)
    palette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
    palette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
    palette.setColor(QPalette.ColorRole.HighlightedText, Qt.GlobalColor.black)
    app.setPalette(palette)

    window = SDCombineDatasets()
    window.show()
    sys.exit(app.exec())


import os 
import sys 
import textwrap 

if sys .version_info [0 ]==2 :
    from cStringIO import StringIO 
else :
    from io import StringIO 

PRESCRIPT =textwrap .dedent (
"""\
    import pkg_resources, zipimport, os

    def find_eggs_in_zip(importer, path_item, only=False):
        if importer.archive.endswith('.whl'):
            # wheels are not supported with this finder
            # they don't have PKG-INFO metadata, and won't ever contain eggs
            return

        metadata = pkg_resources.EggMetadata(importer)
        if metadata.has_metadata('PKG-INFO'):
            yield Distribution.from_filename(path_item, metadata=metadata)
        for subitem in metadata.resource_listdir(''):
            if not only and pkg_resources._is_egg_path(subitem):
                subpath = os.path.join(path_item, subitem)
                dists = find_eggs_in_zip(zipimport.zipimporter(subpath), subpath)
                for dist in dists:
                    yield dist
            elif subitem.lower().endswith(('.dist-info', '.egg-info')):
                subpath = os.path.join(path_item, subitem)
                submeta = pkg_resources.EggMetadata(zipimport.zipimporter(subpath))
                submeta.egg_info = subpath
                yield pkg_resources.Distribution.from_location(path_item, subitem, submeta)  # noqa: B950

    def _fixup_pkg_resources():
        pkg_resources.register_finder(zipimport.zipimporter, find_eggs_in_zip)
        pkg_resources.working_set.entries = []
        list(map(pkg_resources.working_set.add_entry, sys.path))

    _fixup_pkg_resources()
"""
)


def check (cmd ,mf ):
    m =mf .findNode ("pkg_resources")
    if m is None or m .filename is None :
        return None 

    if m .filename .endswith ("__init__.py"):
        vendor_dir =os .path .join (os .path .dirname (m .filename ),"_vendor")
    else :
        vendor_dir =os .path .join (m .filename ,"_vendor")

    expected_missing_imports ={
    "__main__.__requires__",
    }

    if os .path .exists (vendor_dir ):
        for topdir ,dirs ,files in os .walk (vendor_dir ):
            for fn in files :
                if fn in ("__pycache__","__init__.py"):
                    continue 

                relnm =os .path .relpath (os .path .join (topdir ,fn ),vendor_dir )
                if relnm .endswith (".py"):
                    relnm =relnm [:-3 ]
                relnm =relnm .replace ("/",".")

                if fn .endswith (".py"):
                    mf .import_hook ("pkg_resources._vendor."+relnm ,m ,["*"])
                    expected_missing_imports .add ("pkg_resources.extern."+relnm )
            for dn in dirs :
                if not os .path .exists (os .path .join (topdir ,dn ,"__init__.py")):
                    continue 
                relnm =os .path .relpath (os .path .join (topdir ,dn ),vendor_dir )
                relnm =relnm .replace ("/",".")

                mf .import_hook ("pkg_resources._vendor."+relnm ,m ,["*"])
                expected_missing_imports .add ("pkg_resources.extern."+relnm )

        mf .import_hook ("pkg_resources._vendor",m )

    if sys .version [0 ]!=2 :
        expected_missing_imports .add ("__builtin__")

    return {
    "expected_missing_imports":expected_missing_imports ,
    "prescripts":[StringIO (PRESCRIPT )],
    }



from __future__ import absolute_import 

import marshal 
import os 
import re 
import sys 
import warnings 

from .import _imp as imp 

try :
    unicode 
except NameError :
    unicode =str 


if sys .version_info [0 ]==2 :
    from StringIO import StringIO 
    from StringIO import StringIO as BytesIO 

else :
    from io import BytesIO ,StringIO 


def imp_find_module (name ,path =None ):
    """
    same as imp.find_module, but handles dotted names
    """
    names =name .split (".")
    if path is not None :
        if isinstance (path ,(str ,unicode )):
            path =[os .path .realpath (path )]
    for name in names :
        result =imp .find_module (name ,path )
        if result [0 ]is not None :
            result [0 ].close ()
        path =[result [1 ]]
    return result 


def _check_importer_for_path (name ,path_item ):
    try :
        importer =sys .path_importer_cache [path_item ]
    except KeyError :
        for path_hook in sys .path_hooks :
            try :
                importer =path_hook (path_item )
                break 
            except ImportError :
                pass 
        else :
            importer =None 
        sys .path_importer_cache .setdefault (path_item ,importer )

    if importer is None :
        try :
            return imp .find_module (name ,[path_item ])
        except ImportError :
            return None 

    if hasattr (importer ,"find_module"):
        return importer .find_module (name )
    else :
        spec =importer .find_spec (name )
        if spec is None :
            return None 
        return spec .loader 


def imp_walk (name ):
    """
    yields namepart, tuple_or_importer for each path item

    raise ImportError if a name can not be found.
    """
    warnings .warn (
    "imp_walk will be removed in a future version",DeprecationWarning ,stacklevel =2 
    )

    if name in sys .builtin_module_names :
        yield name ,(None ,None ,("","",imp .C_BUILTIN ))
        return 
    paths =sys .path 
    res =None 
    for namepart in name .split ("."):
        for path_item in paths :
            res =_check_importer_for_path (namepart ,path_item )
            if hasattr (res ,"load_module"):
                if res .path .endswith (".py")or res .path .endswith (".pyw"):
                    fp =StringIO (res .get_source (namepart ))
                    res =(fp ,res .path ,(".py","rU",imp .PY_SOURCE ))
                elif res .path .endswith (".pyc")or res .path .endswith (".pyo"):
                    co =res .get_code (namepart )
                    fp =BytesIO (imp .get_magic ()+b"\0\0\0\0"+marshal .dumps (co ))
                    res =(fp ,res .path ,(".pyc","rb",imp .PY_COMPILED ))

                else :
                    res =(
                    None ,
                    res .path ,
                    (os .path .splitext (res .path )[-1 ],"rb",imp .C_EXTENSION ),
                    )

                break 
            elif isinstance (res ,tuple ):
                break 
        else :
            break 

        yield namepart ,res 
        paths =[os .path .join (path_item ,namepart )]
    else :
        return 

    raise ImportError ("No module named %s"%(name ,))


cookie_re =re .compile (rb"coding[:=]\s*([-\w.]+)")
if sys .version_info [0 ]==2 :
    default_encoding ="ascii"
else :
    default_encoding ="utf-8"


def guess_encoding (fp ):
    for _i in range (2 ):
        ln =fp .readline ()

        m =cookie_re .search (ln )
        if m is not None :
            return m .group (1 ).decode ("ascii")

    return default_encoding 



def _site_packages ():
    import os 
    import site 
    import sys 

    paths =[]
    prefixes =[sys .prefix ]
    if sys .exec_prefix !=sys .prefix :
        prefixes .append (sys .exec_prefix )
    for prefix in prefixes :
        paths .append (
        os .path .join (
        prefix ,"lib","python%d.%d"%(sys .version_info [:2 ]),"site-packages"
        )
        )

    if os .path .join (".framework","")in os .path .join (sys .prefix ,""):
        home =os .environ .get ("HOME")
        if home :

            paths .append (
            os .path .join (
            home ,
            "Library",
            "Python",
            "%d.%d"%(sys .version_info [:2 ]),
            "lib",
            "python",
            "site-packages",
            )
            )


            paths .append (
            os .path .join (
            home ,
            "Library",
            "Python",
            "%d.%d"%(sys .version_info [:2 ]),
            "site-packages",
            )
            )





    sys .__egginsert =len (sys .path )

    for path in paths :
        site .addsitedir (path )


_site_packages ()



import os 


def check (cmd ,mf ):
    m =mf .findNode ("pygame")
    if m is None or m .filename is None :
        return None 

    def addpath (f ):
        return os .path .join (os .path .dirname (m .filename ),f )

    RESOURCES =["freesansbold.ttf","pygame_icon.icns"]
    return {"loader_files":[("pygame",map (addpath ,RESOURCES ))]}



def check (cmd ,mf ):
    m =mf .findNode ("pydoc")
    if m is None or m .filename is None :
        return None 
    refs =[
    "Tkinter",
    "tty",
    "BaseHTTPServer",
    "mimetools",
    "select",
    "threading",
    "ic",
    "getopt",
    "tkinter",
    "win32",
    ]
    for ref in refs :
        mf .removeReference (m ,ref )
    return {}



AUTO_PACKAGES =[


"botocore",
"docutils",
"pylint",

"h5py",





"Crypto",

"sentencepiece",



"imageio_ffmpeg",

"numpy",
"scipy",
"tensorflow",
]


def check (cmd ,mf ):
    to_include =[]
    for python_package in AUTO_PACKAGES :
        m =mf .findNode (python_package )
        if m is None or m .filename is None :
            continue 

        to_include .append (python_package )

    if to_include :
        return {"packages":to_include }
    return None 
